<?php

namespace Litepie\Location\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Litepie\Location\Forms\Country as CountryForm;
use Litepie\Location\Http\Requests\CountryRequest;
use Litepie\Location\Interfaces\CountryRepositoryInterface;
use Litepie\Location\Repositories\Eloquent\Filters\CountryResourceFilter;
use Litepie\Location\Repositories\Eloquent\Presenters\CountryListPresenter;

/**
 * Resource controller class for country.
 */
class CountryResourceController extends BaseController
{

    /**
     * Initialize country resource controller.
     *
     *
     * @return null
     */
    public function __construct(CountryRepositoryInterface $country)
    {
        parent::__construct();
        $this->form = CountryForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('litepie.location.modules'), 'location', guard_url('location'));
        $this->repository = $country;
    }

    /**
     * Display a list of country.
     *
     * @return Response
     */
    public function index(CountryRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(CountryResourceFilter::class)
            ->setPresenter(CountryListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('location::country.names'))
            ->view('location::country.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display country.
     *
     * @param Request $request
     * @param Model   $country
     *
     * @return Response
     */
    public function show(CountryRequest $request, CountryRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('location::country.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('location::country.show')
            ->output();
    }

    /**
     * Show the form for creating a new country.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(CountryRequest $request, CountryRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('location::country.name'))
            ->view('location::country.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new country.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CountryRequest $request, CountryRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('location::country.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('location/country/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/location/country'))
                ->redirect();
        }

    }

    /**
     * Show country for editing.
     *
     * @param Request $request
     * @param Model   $country
     *
     * @return Response
     */
    public function edit(CountryRequest $request, CountryRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('location::country.name'))
            ->view('location::country.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the country.
     *
     * @param Request $request
     * @param Model   $country
     *
     * @return Response
     */
    public function update(CountryRequest $request, CountryRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('location::country.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('location/country/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('location/country/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the country.
     *
     * @param Model   $country
     *
     * @return Response
     */
    public function destroy(CountryRequest $request, CountryRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('location::country.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('location/country/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('location/country/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
